//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	NumericFunctors.h: --- operator functions in numerical context
//
//	DESCRIPTION:
//		Definition of prototypes of all operator classes instantiated
//		by the yacc/bison generated parser. These are:
//
//			Identifier			: an identifier
//			Literal				: a literal, i.e., constant
// 
//			Add/Sub/Mul/Div		: basic arithmetic '+', '-', '*'. '/'
//			Expo				: exponentiation '**' (for the Fortran lovers
//								  among us)
//			And/Or/Xor			: bitsise operations
//			Lshift/Rshift		; bit shifting
//			Bitcompl			: one's complement
//			Minus				: unary minus
//
//			Not/AndIf/OrIf		: logicals operator
//			Lt,Gt,Le,Ge,Eq,Ne	: relational operators: <, >, <=, >=, ==, !=
//
//			Fabs				: absolute value
//			Fsin, Fcos, Ftan, 	: the trigs
//			Asin, Acos, Atan,
//			Atan2				: the inverse trigs
//			Fsinh, Fcosh, Ftanh : hyperbolic trigs
//
//			Fexp				: exp() function
//			Flog, Flog10		: natural/base10 logarithm
//			Fsqrt				: the square root
//
//			Modf				: fractional part of arg
//			Pow					: power function
//			Ceil				: smallest integral value not less than arg
//			Floor				: largest integral value not greater than arg
//			Fmod				: floating point remainder x/y
//			Fint				: integer part of arg
//
//			Fcircle				: point-in-circle test
//			Fring				: point-in-ring test
//			Fbox				: point-in-box test
//			Fellipse			: point-in-ellipse test
//			Fpolygon			: point-in-polygon test
//
//			NanOp				: NAN tests
//
//	CHANGE HISTORY:
//		UL, 1998-01-05: 1.0.0: - first version
//		UL, 1998-05-19: 2.0.0: - major overhaul to bring it in line with
//								 other changed leading to selectlib v2.0
//		UL, 1998-06-18: 2.1.0: - added interval-related operator functions
//							     IntEq, IntGt, etc.
//		UL, 1999-08-16: 2.2.0: - added hyperbolic trigs +
//								 pow/ceil/floor/fmod/modf/int
//
//	AUTHOR:
//		U. Lammers (UL), Wed Aug 27 11:48:12 MET DST 1997
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/selcore/NumericFunctors.h,v 1.2 2002/05/24 16:00:10 ulammers Exp $
///

#ifndef NUMERIC_FUNCTORS_H
#define NUMERIC_FUNCTORS_H

#include "Selector.h"
#include "SelectError.h"

#include <vector>

#define BEGINCLASS(a)					\
class a : public OperatorNode {			\
public:									\
	a();								\
	~a();

#define FINAL							\
	void finalize();

#define ENDCLASS };

#define OPCLASS(a)		BEGINCLASS(a) ENDCLASS
#define OPCLASS1(a)		BEGINCLASS(a) a(const OpNode); ENDCLASS
#define OPCLASS2(a)		BEGINCLASS(a) a(const OpNode, const OpNode); ENDCLASS
#define OPCLASS3(a)		BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode);						\
						ENDCLASS
#define OPCLASS3_FINAL(a)													\
						BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode);						\
						FINAL												\
						ENDCLASS
#define OPCLASS4(a)		BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode, const OpNode);		\
						ENDCLASS
#define OPCLASS5(a)		BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode);						\
						ENDCLASS
#define OPCLASS6(a)		BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode, const OpNode);		\
						ENDCLASS
#define OPCLASS7(a)		BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode);						\
						ENDCLASS
#define OPCLASS10(a)	BEGINCLASS(a) a(const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode, const OpNode,			\
										const OpNode, const OpNode);		\
						ENDCLASS


//
//	operator classes
//
OPCLASS2(AndIf);
OPCLASS2(OrIf);
OPCLASS1(Not);

OPCLASS2(Lt);
OPCLASS2(Le);
OPCLASS2(Gt);
OPCLASS2(Ge);
OPCLASS2(Eq);
OPCLASS2(Ne);

OPCLASS2(Add);
OPCLASS2(Sub);
OPCLASS2(Mul);
OPCLASS2(Div);
OPCLASS2(Modulo);
OPCLASS2(Expo);
OPCLASS1(Uminus);
OPCLASS2(And);
OPCLASS2(Or);
OPCLASS2(Xor);
OPCLASS2(Lshift);
OPCLASS2(Rshift);
OPCLASS1(Bitcompl);

OPCLASS1(Fabs);
OPCLASS1(Fsin);
OPCLASS1(Fcos);
OPCLASS1(Ftan);
OPCLASS1(Fsinh);
OPCLASS1(Fcosh);
OPCLASS1(Ftanh);
OPCLASS1(Arcsin);
OPCLASS1(Arccos);
OPCLASS1(Arctan);
OPCLASS2(Arctan2);

OPCLASS1(Fexp);
OPCLASS1(Flog);
OPCLASS1(Flog10);
OPCLASS1(Fsqrt);
OPCLASS1(Modf);
OPCLASS2(Pow);
OPCLASS1(Floor);
OPCLASS1(Ceil);
OPCLASS1(Fint);
OPCLASS2(Fmod);

OPCLASS4(Fpoint);
OPCLASS6(Fline);
OPCLASS5(Fcircle);
OPCLASS6(Fring);
OPCLASS7(Fellipse);
OPCLASS10(Frellipse);
OPCLASS7(Fbox);
OPCLASS7(Frectangle);
OPCLASS3_FINAL(Fpolygon);
OPCLASS6(Fsector);
OPCLASS7(Fdiamond);

OPCLASS3(Fnear);
OPCLASS3(Ifthenelse);

OPCLASS(TrueNode);
OPCLASS(FalseNode);

OPCLASS1(NanOp);

//
//	Interval related operator functions
//
OPCLASS(IntTrue);
OPCLASS(IntFalse);
OPCLASS1(IntEq);
OPCLASS1(IntLe);
OPCLASS1(IntLt);
OPCLASS1(IntGe);
OPCLASS1(IntGt);
OPCLASS2(IntCC);
OPCLASS2(IntOC);
OPCLASS2(IntCO);
OPCLASS2(IntOO);
OPCLASS(InInterval);

#endif
