//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	VectorFunctors.h: --- vector operator functions in numerical context
//
//	DESCRIPTION:
//		Definition of prototypes of all vector operator classes instantiated
//		by the yacc/bison generated parser. These are:
//
//	CHANGE HISTORY:
//		UL, 2002-05-22: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Wed May 22 17:23:26 MEST 2002
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/selcore/VectorFunctors.h,v 1.2 2009/06/26 15:51:59 aibarra Exp $
///

#ifndef VECTOR_FUNCTORS_H
#define VECTOR_FUNCTORS_H

#include "Selector.h"
#include "SelectError.h"
#include "NumericFunctors.h"

#include "Celestial.h"

class VectorNode : public CartesianVector, public OperatorNode {
public:
	VectorNode() : eval(&VectorNode::evalConst) {}
	~VectorNode() {}

	typedef CartesianVector &(VectorNode::*MemPtr)();
	MemPtr eval;
	/*
	 * AI gcc4 porting
	 * evalVariable() was protected, now it is public.
	 */
	virtual CartesianVector &evalVariable() { return standardEvalVariable(); }


protected:
	CartesianVector &evalConst() { return *this; }

	CartesianVector &standardEvalVariable() {
		unsigned N = nChildren();
		double *v = data();
		for (unsigned n=0; n<N; ++n)
			v[n] = child(n)->arith(child(n));
		return *this;
	}
};

#define BEGINVECCLASS(a)				\
class a : public VectorNode {			\
public:									\
	a() {}								\
	CartesianVector &evalVariable();	\
	~a() {}

#define ENDVECCLASS };

#define VECCLASS1(a)	BEGINVECCLASS(a) a(const OpNode); ENDVECCLASS
#define VECCLASS2(a)	BEGINVECCLASS(a) a(const OpNode, const OpNode);		\
						ENDVECCLASS
#define VECCLASS3(a)	BEGINVECCLASS(a) a(const OpNode, const OpNode,		\
										   const OpNode);					\
						ENDVECCLASS

//
//	operator classes
//
VECCLASS3(Vector)
VECCLASS3(UnitVector)
VECCLASS2(SkyVector)
VECCLASS2(VectorAdd)
VECCLASS2(VectorSub)
VECCLASS2(VectorScalarMult)
VECCLASS2(VectorScalarDiv)
VECCLASS2(CrossProd)
VECCLASS1(VectorMinus)

OPCLASS1(VectorNorm)
OPCLASS2(VectorComponent)
OPCLASS2(ScalarProduct)
OPCLASS3(ConeFilter)

#undef BEGINVECCLASS
#undef ENDVECCLASS
#undef VECCLASS1
#undef VECCLASS2
#undef VECCLASS3

#endif
