//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	DalTableDescriptor.h: --- a TableDescriptor with the Dal
//
//	DESCRIPTION:
//		Some table inquiry functions
//
//	CHANGE HISTORY:
//		UL, 2000-06-30: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Thu Jun 29 16:21:51 MET DST 2000
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/service/DalTableDescriptor.h,v 1.3 2014/04/15 15:01:34 aibarra Exp $
///

#ifndef DAL_TABLE_DESCRIPTOR_H
#define DAL_TABLE_DESCRIPTOR_H

#include "Dal.h"
#include <string>

class DalColumnDescriptor : public ColumnDescriptor {
public:
	DalColumnDescriptor(const Column *column) : _column(column) {}
	bool isInteger() const { Dal::DataType type =  _column->dataType();
							 return type == Dal::Int8 ||
									type == Dal::Uint16 ||
									type == Dal::Int16 ||
									type == Dal::Uint32 ||
									type == Dal::Int32; }
	bool isReal() const { Dal::DataType type = _column->dataType();
						  return type == Dal::Real32 || type == Dal::Real64; }
	bool isBool() const { return _column->dataType() == Dal::Bool; }
	bool nullDefined() const { return _column->nullDefined(); }
	long nullValue() const { return _column->intNullValue(); }
	unsigned length() const { return _column->elements(); }

private:
	const Column *_column;
};

class DalTableDescriptor : public TableDescriptor {
public:
	DalTableDescriptor(const Table *table) : _table(table) {}
	//gcc4.8.1 compatibility
	virtual ~DalTableDescriptor(){}

	std::string	name() const { return _table->name(); }
	std::string	setName() const { return _table->parent()->name(); }
	bool	hasColumn(const std::string &name) const
	{
		return _table->hasColumn(name);
	}

	ColumnDescriptor *column(const std::string &name) const
	{
		return new DalColumnDescriptor(_table->column(name));
	}
	const Table *table() const { return _table; }

private:
	const Table *_table;
};

#endif
