//
//	exprparse: check if an expression adheres to the grammar and evalulate
//			   it if possible
//
//	UL, Thu Feb 19 11:00:26 MET 1998
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/test/exprparse.cc,v 1.8 2002/12/17 17:48:12 jbakker Exp $
//

#include <cassert>
#include <string>
#include <iostream>
#include "Evaluator.h"

int main(int argc, char *argv[])
{
	if (argc != 2) {
		std::cerr<<"Usage: "<<argv[0]<<" <expr>\n";
		return 1;
	}

	Evaluator evaluator(argv[1]);	//	construct evaluator

	//	check identifiers
	NumericSymbols idents = evaluator.numericIdentifiers();
	unsigned nIds = idents.name.size();
	if (nIds) {
		std::cout<<"List of numeric identifiers in expression is:\n";
		for (unsigned n=0; n<nIds; ++n)
			std::cout<<idents.name[n]<<" ";
		std::cout<<std::endl;
	} else
		std::cout<<"Expression contains no numeric identifiers\n";

	StringSymbols sidents = evaluator.stringIdentifiers();
	unsigned nsIds = sidents.name.size();
	if (nsIds) {
		std::cout<<"List of string identifiers in expression is:\n";
		for (unsigned n=0; n<nsIds; ++n)
			std::cout<<sidents.name[n]<<" ";
		std::cout<<std::endl;
	} else
		std::cout<<"Expression contains no std::string identifiers\n";

	//	row number id given
	std::cout<<"Expression contains "<<(evaluator.rowNumber() ? "" : "no ")
		<<"row number identifier\n";
	std::cout<<"Expression is ok (type: "
		<<(evaluator.isLogic() ? "logic" : "arithmetic")<<") ";
	bool res = true;
	if (!nsIds && !nIds) {
		//	no symbols - let's evalulate
		std::cout<<" and evaluates to: ";
		if (evaluator.isLogic()) {
			res = evaluator.evalLogic();
	  		std::cout<<"`"<<(res ? "true" : "false")<<"'";
		} else
			std::cout<<evaluator.evalArith();
	}
	std::cout<<".\n";

	return res ? 0 : 1;
} // main()
