//
//	stringfiltertest: test the filtering on string columns
//
//	UL, Fri Jun 30 13:56:12 MET DST 2000
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/test/stringfiltertest.cc,v 1.7 2009/06/26 15:51:59 aibarra Exp $
//

#include <cassert>
#include <iostream>
#include "DataFilter.h"
#include <cstring>

int main()
{
	const unsigned NROWS = 8;
	const unsigned LENA = 10;
	const unsigned LENB = 15;
	const std::string qualColName = "FLAGS";
	const std::string dsname = "stringcoltest.ds";

	//
	//	create a test set
	//
	Table *table = dataSetServer->open(dsname, DataSet::Create)->
										addTable("STRING", NROWS);
	std::vector<unsigned long> size(1);
	size[0] = LENA;
	Column *a = table->addColumn("ACOL", Dal::DString, "", "", size);
	size[0] = LENB;
	Column *b = table->addColumn("BCOL", Dal::DString, "", "", size);

	ColumnData *cda = a->data(),
			   *cdb = b->data();
	char *ca = cda->stringData(),
		 *cb = cdb->stringData();

	std::string sa, sb;
	for (unsigned i=0; i<NROWS; ++i) {
		std::strncpy(ca+cda->position(i, 0), sa.c_str(), LENA);
		std::strncpy(cb+cdb->position(i, 0), sb.c_str(), LENB);
		sa += 'a'+i;
		sb += 'b'+i;
	}

	delete cda;
	delete cdb;
	dataSetServer->close(table->parent());

	table = dataSetServer->open(dsname, Dal::Modify)->table(0);
	//
	//	tests
	//
	//	  ACOL       BCOL
	//	1
	//	2 a          b
	//	3 ab         bc
	//	4 abc        bcd
	//	5 abcd       bcde
	//	6 abcde      bcdef
	//	7 abcdef     bcdefg
	//	8 abcdefg    bcdefgh
	//
	DataFilter filter01("ACOL==\"\"");
//	The following should give 1 but no rows are being selected - this
//	is due to an apparent cfitsio bug; if the value is '' cfitsio
//	returns a null-terminated single blank ' '
//	assert(filter01.doSelect(*table, qualColName, 0) == 1);

	//	row 4
	DataFilter filter02("ACOL==\"abc\"");
	assert(filter02.doSelect(*table, qualColName, 0) == 1);

	//	rows 4, 5, 6, 7, 8
	DataFilter filter03("ACOL[1:2]==\"bc\"");
	assert(filter03.doSelect(*table, qualColName, 0) == 5);

	//	rows 3, 4, 5, 6, 7, 8
	DataFilter filter04("(ACOL+\"c\")[1:2]==\"bc\"");
	assert(filter04.doSelect(*table, qualColName, 0) == 6);

	//	row 1
	DataFilter filter05("ACOL==BCOL");
	assert(filter05.doSelect(*table, qualColName, 0) == 1);

	//	rows 3, 4, 5, 6, 7, 8
	DataFilter filter06("ACOL[1:]==BCOL[:#ROW-3]");\
	assert(filter06.doSelect(*table, qualColName, 0) == 6);

	dataSetServer->close(table->parent());

	std::cout<<"All tests successfully passed.\n";
	return 0;
}
